
// Pre-calculate some time constants
var msPerSecond = 1000;
var sPerMinute  = 60;
var sPerHour    = 60 * sPerMinute;
var sPerDay     = 24 * sPerHour;


// JavaScript interval timer to update clock
var updateTimerInterval;

// Clock registers
var currentTime;
var remainingSeconds = 0;
var remainingTime = new Array(4);

var countdownTarget = null;

// Flag for whether we've reached/passed the target time
var isTargetReached = false;

// counter of unsuccessfull submits
var submitFails = 0;
// max number of attempts (after 1 min)
var maxSubmitFails = 5; 
var xmlRequestTimeout = 10000; //ms
var submitBatteryUnitTimeout = null;
var submitBatteryHealthTimeout = null;
 
var baseURL = "http://www.macbookbatterydatabase.com/";
var lastSubmit = null;

function setLastSubmit(date){
    lastSubmit = date;
    // Values you provide
    var textFieldToChange = document.getElementById("lastSubmitDateText");	// replace with ID of text field
    if (date)
        var dateText = date.format(dashcode.getLocalizedString("d/m/yyyy HH:MM:ss"));
    else
        var dateText = dashcode.getLocalizedString("Unknown");
    // Text field code
    textFieldToChange.innerText = dateText;

}

//
// Function: setNextCountdownTarget()
// Find the next event in the event list and set the timer
//
function setNextCountdownTarget(date)
{
    updateCurrentTime();

    if (date){
        countdownTarget = date;
        isTargetReached = false;
    } else {
        if (getSubmitEnabled()){
            countdownTarget = new Date();
            countdownTarget.setTime(countdownTarget.getTime() + getSubmitInterval());
            isTargetReached = false;
        } else {
            countdownTarget = null;
        }
    }

    if (countdownTarget)
        var dateText = countdownTarget.format(dashcode.getLocalizedString("d/m/yyyy HH:MM:ss"));
    else
        var dateText = dashcode.getLocalizedString("Unknown");

    // Text field code
    var textFieldToChange = document.getElementById("nextSubmitDateText");	// replace with ID of text field
    textFieldToChange.innerText = dateText;
    
}


//
// Function: ForAlarm()
// Determine whether the target has been reached, and if so, call the alarm hook.
//
function checkForAlarm()
{
    if (remainingSeconds <= 0) {
        // We're there
        if (isTargetReached == false) {
            // Make sure we only get here once
            isTargetReached = true;
            // Call the alarm hook
            doSubmitNow(null);
            // Start countdown to the next event
            setNextCountdownTarget(null);
        }
    }
}

//
// Function: updateCurrentTime()
// Store the current time in the clock's internal state
//
function updateCurrentTime()
{
    currentTime = new Date();
}

//
// Function: calculateRemainingTime()
// Determine the amount of time remaining to the current target
// Result is stored in the remainingTime array as [days, hours, minutes, seconds]
//
function calculateRemainingTime()
{
    if (!countdownTarget) return null;

    // Start by getting the current date
    updateCurrentTime();
    // Clear excessive precision
    currentTime.setMilliseconds(0);

    // Number of seconds between now and target
    remainingSeconds = Math.floor((countdownTarget.getTime() - currentTime.getTime()) / msPerSecond);
    if (remainingSeconds <= 0)
            remainingSeconds = 0;
    
    // Calculate days
    var remainingDays = Math.floor(remainingSeconds / sPerDay);
    // And take remainder
    var leftoverSeconds = remainingSeconds - remainingDays * sPerDay;
    // Same for hours, minutes, and seconds
    var remainingHours = Math.floor(leftoverSeconds / sPerHour);
    leftoverSeconds -= remainingHours * sPerHour;
    var remainingMinutes = Math.floor(leftoverSeconds / sPerMinute);
    leftoverSeconds -= remainingMinutes * sPerMinute;

    remainingTime[0] = remainingDays;
    remainingTime[1] = remainingHours;
    remainingTime[2] = remainingMinutes;
    remainingTime[3] = leftoverSeconds;
}


//
// Function: updateTimer()
// The main loop called by our timer interval.
// Update the clock, display the new time, and run the alarm if the target is reached.
//
function updateTimer()
{
    calculateRemainingTime();
    checkForAlarm();
}


function getSubmitEnabled(){

    return document.getElementById('submitEnabledCheckbox').checked;

}

function getSubmitInterval(){

    return document.getElementById('submitIntervalPopup').value * sPerHour * msPerSecond;

}

function setSelected(oSel, val){
    var len = oSel.options.length;
    for(var i=0; i<len; i++){
        if (oSel.options[i].value == val){
            oSel.selectedIndex = i;
            break;
        }
    } 
}


function setSubmitInterval(value){

    setSelected(document.getElementById('submitIntervalPopup'), value / sPerHour / msPerSecond);
    submitIntervalChanged(null);
}

function setSubmitEnabled(value){

    document.getElementById('submitEnabledCheckbox').checked = value;
    submitEnabledChanged(null);

}

//
// Function: startDisplayUpdateTimer()
// Start the interval timer to update the countdown once a second
//
function startUpdateTimer()
{
    updateTimer();

    if (!updateTimerInterval && getSubmitEnabled())
        updateTimerInterval = setInterval(updateTimer, 1000);
}

//
// Function: stopDisplayUpdateTimer()
// Remove the interval timer
//
function stopUpdateTimer()
{
    if (updateTimerInterval) {
        clearInterval(updateTimerInterval);
        updateTimerInterval = null;
    }
}


function loadPreferences(){
    if (window.widget) {
        setLastSubmit(new Date(widget.preferenceForKey('lastSubmit')));
        setSubmitInterval(widget.preferenceForKey('submitInterval'));
        setSubmitEnabled(widget.preferenceForKey('submitEnabled'));
        setNextCountdownTarget(new Date(widget.preferenceForKey('countdownTarget')));
    }
}

function savePreferences(){
    if (window.widget) {
        widget.setPreferenceForKey(countdownTarget.getTime(), 'countdownTarget');
        widget.setPreferenceForKey(lastSubmit.getTime(), 'lastSubmit');
        widget.setPreferenceForKey(getSubmitEnabled(), 'submitEnabled');
        widget.setPreferenceForKey(getSubmitInterval(), 'submitInterval');      
    }
}

function correctLocalization(){

    document.getElementById("intervalDay").innerHTML = dashcode.getLocalizedString("Day");
    document.getElementById("intervalWeek").innerHTML = dashcode.getLocalizedString("Week");

}

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();
    correctLocalization();
    loadPreferences();
    if (!countdownTarget)
        setNextCountdownTarget(null);
    startUpdateTimer();

    document.getElementById("updateBox").style.display = "none";
    if (widget.preferenceForKey('updateAvailable') === true) {
        document.getElementById("updateBox").style.display = "block";
    }
    
    checkForUpdates();
    
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    stopUpdateTimer();
    savePreferences();
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop timer to prevent CPU usage
   // stopUpdateTimer();
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Re-start timer
    startUpdateTimer();
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}


function doShowMyStats(event)
{
    var params = getBatteryUnit();

    var websiteURL = baseURL + "battery/" + encodeURIComponent(params['battery_id']);	// replace with the website URL to show

    // Show website code
    widget.openURL(websiteURL);

}


function doDownloadUpdate(event)
{
    var websiteURL = baseURL + "widget/download";	// replace with the website URL to show

    widget.setPreferenceForKey(false, 'updateAvailable');      
    document.getElementById("updateBox").style.display = "none";

    // Show website code
    widget.openURL(websiteURL);
}

function checkForUpdates()
{

    var feedURL = baseURL + "widget/version";

    //alert(feedURL);
    var onloadHandler = function() { xmlCheckLoaded(xmlRequest); };	// The function to call when the feed is loaded; currently calls the XMLHttpRequest load snippet

    // XMLHttpRequest setup code
    var xmlRequest = new XMLHttpRequest();
    xmlRequest.onload = onloadHandler;
    xmlRequest.open("GET", feedURL);
    xmlRequest.setRequestHeader("Cache-Control", "no-cache");
    xmlRequest.send(null);

}

// Called when an XMLHttpRequest loads a feed; works with the XMLHttpRequest setup snippet
function xmlCheckLoaded(xmlRequest) 
{
	if (xmlRequest.status == 200) {
//    alert( "OK:"+xmlRequest.responseText);
//alert(xmlRequest.responseText.toString() > getCurrentWidgetVersion().toString());
        if (xmlRequest.responseText.toString() > getCurrentWidgetVersion().toString()){
            widget.setPreferenceForKey(true, 'updateAvailable');
            document.getElementById("updateBox").style.display = "block";
        }
	}
}


function getCurrentWidgetVersion()
{
    var infoPlist = new XMLHttpRequest(); 
    infoPlist.open("GET", "Info.plist", false); 
    infoPlist.send(null); 
    var infoXML = infoPlist.responseXML;
    var nodes = infoXML.getElementsByTagName("dict")[0].childNodes;
    var key_value = "";
    for (var i=0; i < nodes.length; i++) { 
      if (nodes[i].nodeType == 1 && nodes[i].tagName.toLowerCase() == "key" && nodes[i].firstChild.data == "CFBundleVersion") { 
         if (nodes[i+2].tagName.toLowerCase() != "array") { 
            key_value = nodes[i+2].firstChild.data; 
         } else { 
            key_value = new Array(); 
            var ar_nodes = nodes[i+2].childNodes; 
            for (var j=0; j < ar_nodes.length; j++) { 
               if (ar_nodes[j].nodeType == 1) key_value.push(ar_nodes[j].firstChild.data); 
            } 
         } 
         break; 
      } 
   } 
   return key_value; 
}


function xmlRequestFailed(){

    alert("Error sending data!");
    if (submitFails < maxSubmitFails) {
        setNextCountdownTarget(new Date((new Date()).getTime() + 60 * msPerSecond));
        submitFails++;
    } else {
        submitFails = 0;
    }

}

// Called when an XMLHttpRequest loads a feed; works with the XMLHttpRequest setup snippet
function xmlHealthLoaded(xmlRequest, submitUnit) 
{
    clearTimeout(submitBatteryHealthTimeout);
	if (xmlRequest.status == 200) {
//    alert( "OK:"+xmlRequest.responseText);
        setLastSubmit(new Date());
        if (parseInt(xmlRequest.responseText) == -1 && submitUnit)
	    	submitBatteryUnit();
	}
	else {
        xmlRequestFailed();
	}
}

// Called when an XMLHttpRequest loads a feed; works with the XMLHttpRequest setup snippet
function xmlUnitLoaded(xmlRequest) 
{
    clearTimeout(submitBatteryUnitTimeout);
	if (xmlRequest.status == 200) {
//    alert( "OK:"+xmlRequest.responseText);
        if (parseInt(xmlRequest.responseText) == 0)
	    	submitBatteryHealth(false);
	}
	else {
        xmlRequestFailed();
	}
}

function getXMLPlistValue(xmlDocument, keyName){

    return document.evaluate('//key[.="' + keyName + '"]/following-sibling::*[1]/text()', xmlDocument, null, XPathResult.STRING_TYPE, null).stringValue; 

}

function getKeyValue(treeData, keyName){

    var re = new RegExp("\"" + keyName + "\" = <?\"?([0-9a-zA-Z-_.!@#]*)\"?>?");
    var match = re.exec(treeData);
    if (match != null) {
        return match[1];
    }
}

function decimalToHex(d, padding) {
    var hex = Number(d).toString(16);
    padding = typeof (padding) === "undefined" || padding === null ? padding = 2 : padding;

    while (hex.length < padding) {
        hex = "0" + hex;
    }

    return hex;
}


function getBatteryId(treeData){

    var battery_id = getKeyValue(treeData, "BatterySerialNumber");
    if (typeof(battery_id) === "undefined" || battery_id === null || battery_id === ""){
        battery_id = getKeyValue(treeData, "Manufacturer") + "-" + getKeyValue(treeData, "DeviceName") + "-" + decimalToHex(getKeyValue(treeData, "ManufactureDate"), 4);
        var firmware_serial_number = getKeyValue(treeData, "FirmwareSerialNumber");
        var serial_number = getKeyValue(treeData, "SerialNumber");
        var serial = getKeyValue(treeData, "Serial");
        if (typeof(firmware_serial_number) !== "undefined" && firmware_serial_number !== null && firmware_serial_number !== "") {
            if (isNaN(Number(firmware_serial_number))) {
                battery_id += "-" + firmware_serial_number;
            } else {
                battery_id += "-" + decimalToHex(firmware_serial_number, 4);
            }
        } else if (typeof(serial_number) !== "undefined" && serial_number !== null && serial_number !== "") {
            if (isNaN(Number(serial_number))) {
                battery_id += "-" + serial_number;
            } else {
                battery_id += "-" + decimalToHex(serial_number, 4);
            }
        } else if (typeof(serial) !== "undefined" && serial !== null && serial !== "") {
            if (isNaN(Number(serial))) {
                battery_id += "-" + serial;
            } else {
                battery_id += "-" + decimalToHex(serial, 4);
            }
        }
    }
    return battery_id;

}

function getModelId(){
    var result = widget.system('/usr/sbin/sysctl -n hw.model', null);
    return result.outputString.trim();
}

// from ioregs
function getBatteryHealth(){

    var result = widget.system('/usr/sbin/ioreg -w 0 -r -b -n AppleSmartBattery', null);
    var macInfo = {
        "cycle_count" : "CycleCount", 
        "avg_time_to_empty" : "AvgTimeToEmpty",
        "max_capacity" : "MaxCapacity"
    };
    for (var param in macInfo) {
        macInfo[param] = getKeyValue(result.outputString, macInfo[param]);
        if (typeof(macInfo[param]) === "undefined" || macInfo[param] === null || macInfo[param] === "")
            macInfo[param] = "";
    }

    macInfo["battery_id"] = getBatteryId(result.outputString);
    macInfo["model_id"] = getModelId();

    return macInfo;

}

function getBatteryUnit(){
    var result = widget.system('/usr/sbin/ioreg -w 0 -r -b -n AppleSmartBattery', null);
    var macInfo = {
        "battery_serial_number" : "BatterySerialNumber", 
        "manufacturer" : "Manufacturer", 
        "firmware_serial_number" : "FirmwareSerialNumber", 
        "manufacture_date" : "ManufactureDate", 
        "device_name" : "DeviceName", 
        "manufacturer_data" : "ManufacturerData", 
        "design_capacity" : "DesignCapacity"
    };
    for (var param in macInfo) {
        macInfo[param] = getKeyValue(result.outputString, macInfo[param]);
        if (typeof(macInfo[param]) === "undefined" || macInfo[param] === null || macInfo[param] === "")
            macInfo[param] = "";
    }

    macInfo["battery_id"] = getBatteryId(result.outputString);
    macInfo["model_id"] = getModelId();

    return macInfo;

}


/*
// from system_profiler
function getBatteryHealth(){

//    var result = widget.system('/usr/sbin/system_profiler -xml SPHardwareDataType SPPowerDataType', null);
    var result = widget.system('/usr/sbin/system_profiler -xml  SPPowerDataType', null);

    parser=new DOMParser();
    var    xmlDocument=parser.parseFromString(result.outputString,"text/xml");
    var prefix = "sppower_battery_";
    var macInfo = {
        "serial_number" : "", 
        "cycle_count" : "", 
        "max_capacity" : ""
    };
    for (var param in macInfo)
        macInfo[param] = getXMLPlistValue(xmlDocument, prefix + param);

    return macInfo;

}

function getBatteryUnit(){

//    var result = widget.system('/usr/sbin/system_profiler -xml SPHardwareDataType SPPowerDataType', null);
    var result = widget.system('/usr/sbin/system_profiler -xml  SPPowerDataType', null);

    parser=new DOMParser();
    var    xmlDocument=parser.parseFromString(result.outputString,"text/xml");
    var prefix = "sppower_battery_";
    var macInfo = {
        "serial_number" : "",
        "manufacturer" : "",
        "hardware_revision" : "",
        "firmware_version" : "",
        "cell_revision" : "",
        "device_name" : ""
    };
    for (var param in macInfo)
        macInfo[param] = getXMLPlistValue(xmlDocument, prefix + param);

    return macInfo;

}
*/

function doSubmitNow(event){
    submitBatteryHealth();
}

function submitBatteryHealth(submitUnit)
{
    //default is submitunit==true
    submitUnit = typeof submitUnit !== 'undefined' ? submitUnit : true;

    if (submitUnit)
        checkForUpdates();

    var health = getBatteryHealth();
    var feedURL = baseURL + "widget/?submit_type=battery_health&widget_version=" + encodeURIComponent(getCurrentWidgetVersion());
    for(var param in health)
        feedURL += "&" + param + "=" + encodeURIComponent(health[param]); 
    //alert(feedURL);
    var onloadHandler = function() { xmlHealthLoaded(xmlRequest, submitUnit); };	// The function to call when the feed is loaded; currently calls the XMLHttpRequest load snippet

    // XMLHttpRequest setup code
    var xmlRequest = new XMLHttpRequest();
    xmlRequest.onload = onloadHandler;
    xmlRequest.open("GET", feedURL);
    xmlRequest.setRequestHeader("Cache-Control", "no-cache");
    xmlRequest.send(null);
    submitBatteryHealthTimeout = setTimeout(xmlRequestFailed, xmlRequestTimeout);
    
}

function submitBatteryUnit()
{

    var unit = getBatteryUnit();
    var feedURL = baseURL + "widget/?submit_type=battery_unit&widget_version=" + encodeURIComponent(getCurrentWidgetVersion());
    for(var param in unit)
        feedURL += "&" + param + "=" + encodeURIComponent(unit[param]); 
   // alert(feedURL);
    var onloadHandler = function() { xmlUnitLoaded(xmlRequest); };	// The function to call when the feed is loaded; currently calls the XMLHttpRequest load snippet

    // XMLHttpRequest setup code
    var xmlRequest = new XMLHttpRequest();
    xmlRequest.onload = onloadHandler;
    xmlRequest.open("GET", feedURL);
    xmlRequest.timeout = xmlRequestTimeout;
    xmlRequest.ontimeout = xmlRequestFailed;
    xmlRequest.setRequestHeader("Cache-Control", "no-cache");
    xmlRequest.send(null);
    submitBatteryUnitTimeout = setTimeout(xmlRequestFailed, xmlRequestTimeout);

}


function submitEnabledChanged(event)
{
    if (!getSubmitEnabled()) {
        setNextCountdownTarget(null);
        stopUpdateTimer();
    } else {
        setNextCountdownTarget(null);
        startUpdateTimer();
    }
}


function submitIntervalChanged(event)
{
        setNextCountdownTarget(null);    
}


function changeSubmitEnabled(event)
{
    setSubmitEnabled(!getSubmitEnabled());
}